; Creates Hotkeys on-the-fly for inserting text into any document or 
; Editing field  Jack Dunning
; https://jacksautohotkeyblog.wordpress.com/

/*
October 12, 2017

In the original version of InstantHotkey running multiple Hotkeys required
loading the script multiple time. This version allows multiple Hotkeys
with only one loading.

This new version of the Instant Hotkey script offers a number of changes:

1. Multiple GUIs running in same script.
2. Easy to integrate with another script or run solo.
3. Uses pseudo array to track GUIs and Menu items.
4. Uses function to create two-deep variable from illegal characters.
5. Short auto-execute for initiation.
6. Click icon or menu item for first Instant Hotkey setup.

This script features a number of scripting techniques.

October 18, 2017

The function for creating two-deep variable from illegal characters gets
replaced with a Loop in the TextAdd: Lable subroutine which matches the
correct Hotkey combination and inserts text.

*/

#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.

/*
This first label acts as the auto-execute section for the script. It initializes the
variable IH_Count which tracks the number of GUIs and Menu items.

At first, only the menu item "Instant Hotkey" appears in the System Tray icon right-click
menu. Once clicked, "Instant Hotkey" becomes the parent menu for the new GUIs and Hotkeys.
Set as the default, a click of the System Tray icon launches a new Instant Hotkey.

Combine with another script by adding "GoSub, InstantHotkey" to the auto-execute section of 
the combo script and "#Include, [path]\InstantHotkeyTwoDeep.ahk" somewhere after the auto-
execute section of the master script.
*/


InstantHotkey:

IH_Count := 0 

Menu, Tray, UseErrorLevel     ; prevents various strange errors from stopping the script
Menu, HotkeyMenu, Add, New Instant Hotkey, AddNewHotkey
Menu, Tray, Add, Instant Hotkeys, AddNewHotkey
Menu, Tray, Icon, InstantHotkey.ico, 1
Menu, Tray, Icon, Instant Hotkeys, InstantHotkey.ico, 1
Menu, HotkeyMenu, Icon, New Instant Hotkey, C:\Windows\System32\shell32.dll, 319
Menu, Tray, Default, Instant Hotkeys
Menu, Tray, Click, 1

Return

/*
The subroutine AddNewHotkey setup a new GUI for creating an Instant Hotkey. It uses
the name "InstantHotkey" with IH_Count concatenated to create a pseudo array for tracking
GUI pop-ups.
*/

AddNewHotkey:

IH_Count++   ; Instant Hotkey GUI number

If IH_Count = 1
   Menu, Tray, Add, Instant Hotkeys, :HotkeyMenu

; Create a GUIName variable for naming new GUIs and act as a two-deep variable.

GUIName := "InstantHotkey" . IH_Count

Gui, %GUIName%: new           ; Create a new GUI
Gui, %GUIName%:Add, Text,, Enter Hotkeys (ALT, SHIFT, and/or CTRL plus Key)
; The following hidden edit field tracks the GUI/menu item number.
Gui, %GUIName%:Add, edit, hidden ys vGuiNum,%IH_Count%
Gui, %GUIName%:Add, Hotkey, vKeyCombo ys Limit1
Gui, %GUIName%:Add, Edit, w400 vTextInsert xs, <Enter Hotkey Text>
Gui, %GUIName%:Add, Button, gSetupKey , Set Key Combo

; The "Insert" option places the new menu item before "New Instant Hotkey"
; which preserves the matchup between the GUI numbers and menu item numbers.

Menu, HotkeyMenu, Insert,New Instant Hotkey,  %GUIName%, ShowHotkey
Menu, HotkeyMenu, Icon, %GUIName%, InstantHotkey.ico, 1
Menu, InsertText, Add, %GUIName%, TrayTextAdd
Menu, InsertText, Icon, %GUIName%, InstantHotkey.ico, 1

; On the first run, attach submenus to Tray menu items.

If IH_Count = 1
  {
   Menu, Tray, Add, Instant Hotkeys, :HotkeyMenu
   Menu, Tray, Insert, Instant Hotkeys, Add Text, :InsertText
   Menu, Tray, Icon, Add Text, InstantHotkey.ico, 1
  }

Gui, Show, , Instant Hotkey %IH_Count%


Return

SetupKey:
Gui, Submit

; A_Gui contains the name of the GUI (e.g. InstantHotkey1). OldHotkey
; becomes a two-deep variable for storing the Hotkey combination 
; OldHotkey => Instant Hotkey variable => Hotkey combination 
; (e.g. OldHotkey value => "InstantHotkey1" value => "^!O")

OldHotkey := %A_Gui%             ; e.g. OldHotkey = value of "Hotkey1"

If OldHotkey !=                  ; Turn off OldHotkey
    Hotkey, %OldHotkey%, Off     ; e.g. %OldHotkey% => Hotkey1 value or "^!o"
If KeyCombo !=
    Hotkey, %KeyCombo%, TextAdd, On

; Create pseudo-array variable for text

If KeyCombo !=
    IH_HotkeyText%GuiNum% := TextInsert

; An ampersand (&) appended to a menu number calls that menu item.

Menu, InsertText, Rename, %GuiNum%& , % TextInsert
Menu, HotkeyMenu, Rename, %GuiNum%& , % KeyCombo . " " .  TextInsert

%A_Gui% := KeyCombo     ; Save Hotkey combination to two-deep variable

Return

TrayTextAdd:
 SendInput, !{Escape}        ; Jump to last window
 SendRaw, %A_ThisMenuItem%
Return

; The Hotkey text insertion subroutine now uses a Loop to match the Hotkey 
; (A_ThisHotkey) to the value of the  saved two-deep variable (e.g. InstantHotkey1,
; InstantHotkey3, etc), then Break Loop when finding a match. This should
; work with any Hotkey combination.

TextAdd:

 Loop, % IH_Count  ; IH_Count equals the number of Instant Hotkey GUIs
   {
     If A_ThisHotkey = % InstantHotkey%A_Index%
       {
         SendRaw, % IH_HotkeyText%A_Index%
         Break       ; Stop looping
       }       
   }

Return

ShowHotkey:
 Gui, InstantHotkey%A_ThisMenuItemPos%:show
Return


; Remove comma to activate Hotkey CTRL+ALT+H for new Instant Hotkey
; ^!h::GoSub, AddNewHotkey